Imports Microsoft.VisualBasic
Imports System
Imports System.ComponentModel
Imports System.ComponentModel.DataAnnotations
Imports System.Windows.Data

Namespace DataFormTest
	<CustomValidation(GetType(EmployeeValidator), "ValidateEmployee")> _
	Public Class Employee
		Inherits ModelBase
		Implements IEditableObject
		Private Const phoneExpression As String = "^\(\d\d\d\) \d\d\d\-\d\d\d\d$"
		Private Const emailExpression As String = "^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$"

		Private privatecache As Employee
		Private Property cache() As Employee
			Get
				Return privatecache
			End Get
			Set(ByVal value As Employee)
				privatecache = value
			End Set
		End Property
		'[Display(AutoGenerateField = false)]

		Private employeeId_Renamed As Integer
		<Display(Name:="Id", Description := "Employee Id"), Key, Editable(False)> _
		Public Property EmployeeId() As Integer
			Get
				Return employeeId_Renamed
			End Get
			Set(ByVal value As Integer)
				If employeeId_Renamed = value Then
					Return
				End If
				employeeId_Renamed = value
				FirePropertyChanged("EmployeeId")
			End Set
		End Property

		Private firstName_Renamed As String
		<Required(ErrorMessage := "Required field"), StringLength(40, ErrorMessage := "Cannot exceed 40"), Display(Name := "First Name", Description := "Employee's first name")> _
		Public Property FirstName() As String
			Get
				Return firstName_Renamed
			End Get
			Set(ByVal value As String)
				If firstName_Renamed = value Then
					Return
				End If
				Dim propertyName = "FirstName"
				Validate(value, propertyName)
				firstName_Renamed = value
				FirePropertyChanged(propertyName)
			End Set
		End Property

		Private lastName_Renamed As String
		<Required(ErrorMessage := "Required field"), StringLength(40, ErrorMessage := "Cannot exceed 40"), Display(Name := "Last Name", Description := "Employee's last name")> _
		Public Property LastName() As String
			Get
				Return lastName_Renamed
			End Get
			Set(ByVal value As String)
				If lastName_Renamed = value Then
					Return
				End If
				Dim propertyName = "LastName"
				Validate(value, propertyName)
				lastName_Renamed = value
				FirePropertyChanged(propertyName)
			End Set
		End Property

		Private annualSalary_Renamed As Decimal
		<Required(ErrorMessage := "Salary is required"), Range(0, 1000000), Display(Name:="Salary $", Description := "Salary of the employee")> _
		Public Property AnnualSalary() As Decimal
			Get
				Return annualSalary_Renamed
			End Get
			Set(ByVal value As Decimal)
				If annualSalary_Renamed = value Then
					Return
				End If
				Dim propertyName = "AnnualSalary"
				Validate(value, propertyName)
				annualSalary_Renamed = value
				FirePropertyChanged(propertyName)
			End Set
		End Property

		Private age_Renamed As Integer
		'[Range(0, 150, ErrorMessage = "Value must be within range")]
		<Required(ErrorMessage := "Age is required"), Display(Name := "Age", Description := "Age of the employee"), CustomValidation(GetType(EmployeeValidator), "ValidateAge")> _
		Public Property Age() As Integer
			Get
				Return age_Renamed
			End Get
			Set(ByVal value As Integer)
				If age_Renamed = value Then
					Return
				End If
				Dim propertyName = "Age"
				Validate(value, propertyName)
				age_Renamed = value
				FirePropertyChanged(propertyName)
			End Set
		End Property

		Private email_Renamed As String
		<Display(Name := "Email", Description := "Email of the employee"), RegularExpression(emailExpression, ErrorMessage := "Please enter a valid email address")> _
		Public Property Email() As String
			Get
				Return email_Renamed
			End Get
			Set(ByVal value As String)
				If email_Renamed = value Then
					Return
				End If
				Dim propertyName = "Email"
				Validate(value, propertyName)
				email_Renamed = value
				FirePropertyChanged(propertyName)
			End Set
		End Property

		Private phone_Renamed As String
		<Display(Name := "Phone", Description := "Employee's phone number (###) ###-####"), RegularExpression(phoneExpression, ErrorMessage := "Phone number must match the format (###) ###-####")> _
		Public Property Phone() As String
			Get
				Return phone_Renamed
			End Get
			Set(ByVal value As String)
				If phone_Renamed = value Then
					Return
				End If
				Dim propertyName = "Phone"
				Validate(value, propertyName)
				phone_Renamed = value
				FirePropertyChanged(propertyName)
			End Set
		End Property

		#Region "Implementation of IEditableObject"

		Public Overrides Sub BeginEdit() Implements IEditableObject.BeginEdit
			Me.cache = New Employee() With {.EmployeeId = Me.EmployeeId, .Age = Me.Age, .AnnualSalary = Me.AnnualSalary, .Email = Me.Email, .FirstName = Me.FirstName, .LastName = Me.LastName, .Phone = Me.Phone}
		End Sub

		Public Overrides Sub CancelEdit() Implements IEditableObject.CancelEdit
			Me.EmployeeId = Me.cache.EmployeeId
			Me.Age = Me.cache.Age
			Me.AnnualSalary = Me.cache.AnnualSalary
			Me.Email = Me.cache.Email
			Me.FirstName = Me.cache.FirstName
			Me.LastName = Me.cache.LastName
			Me.Phone = Me.cache.Phone
		End Sub

		Public Overrides Sub EndEdit() Implements IEditableObject.EndEdit
			Me.cache = Nothing
			'    Validator.ValidateObject(this, new ValidationContext(this, null, null)); // true);
		End Sub

		#End Region
	End Class
End Namespace

